from cpl_discord.service import DiscordBotServiceABC
from discord import Permissions, Colour

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.data_seeder_abc import DataSeederABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.service.level_repository_service import LevelRepositoryService
from modules.level.configuration.level_settings import LevelSettings


class LevelSeeder(DataSeederABC):

    def __init__(self, logger: DatabaseLogger, levels: LevelSettings, level_repo: LevelRepositoryService, servers: ServerRepositoryABC, bot: DiscordBotServiceABC):
        DataSeederABC.__init__(self)

        self._logger = logger
        self._default_levels = levels.levels.order_by_descending(lambda l: l.min_xp)
        self._levels = level_repo
        self._servers = servers
        self._bot = bot

    async def seed(self):
        if self._levels.get_levels().count() > 0:
            return

        for guild in self._bot.guilds:
            server = self._servers.find_server_by_discord_id(guild.id)
            if server is None:
                continue

            levels = self._levels.find_levels_by_server_id(server.server_id)
            if levels is not None and levels.count() > 0:
                continue

            for level in self._default_levels:
                try:
                    if guild.roles.where(lambda r: r.name == level.name).count() == 0:
                        await guild.create_role(name=level.name, colour=Colour(int(level.color, 16)), hoist=False, mentionable=True, permissions=Permissions(level.permissions))

                    self._logger.info(__name__, f'Created default level {level.name}')
                    level.server = server
                    self._levels.add_level(level)
                except Exception as e:
                    self._logger.error(__name__, f'Creating default role failed', e)

        self._logger.debug(__name__, f'Seeded default levels')
